# encoding: utf-8
module Yell #:nodoc:
  module Helpers #:nodoc:
    module Base #:nodoc:

      private

      # stub
      def reset!
      end

      def inspectables
        []
      end

    end
  end
end

